#include <io.h>
#include <stdio.h>
#include <windows.h>

// -------------------------------------

void usage()
{
    printf (
            "Usage: wavmix [-i] <file1.wav> <file2.wav> <newfile.wav>\n"
            "-i : inverse mix\n"
           );
}

int main ( int argc, const char **argv )
{
    FILE *f1, *f2, *fo;
    char h1[44], h2[44];
    const char *name1, *name2, *nameo;
    int error = 0, inverse = 0;

    if ( argc < 4 || argc > 5 ) {
        usage();
        return 0;
    }

    if ( argv[1][0] == '-' && argv[1][1] == 'i' && argv[1][2] == '\0' ) inverse = 1;

    name1 = argv[1+inverse];
    name2 = argv[2+inverse];
    nameo = argv[3+inverse];

    f1 = fopen ( name1, "rb" );
    if ( !f1 ) {
        printf ( "Failed to open '%s'\n", name1 );
        return 1;
    }

    f2 = fopen ( name2, "rb" );
    if ( !f2 ) {
        fclose ( f1 );
        printf ( "Failed to open '%s'\n", name2 );
        return 1;
    }

    fread ( h1, 1, 44, f1 );
    fread ( h2, 1, 44, f2 );

    if ( memcmp (h1, h2, 44) ) {
        fclose ( f1 );
        fclose ( f2 );
        printf ( "Incompatible input files\n" );
        return 1;
    }

    fo = fopen ( nameo, "wb" );
    if ( !fo ) {
        fclose ( f1 );
        fclose ( f2 );
        printf ( "Failed to open '%s'\n", nameo );
        return 1;
    }

    if ( fwrite (h1, 1, 44, fo) < 44 ) {
        fclose ( f1 );
        fclose ( f2 );
        fclose ( fo );
        printf ( "Write error\n" );
        return 1;
    }

    while ( 1 ) {
        unsigned short s1[512], s2[512], so[512];
        size_t ns1 = fread ( s1, 2, 512, f1 );
        size_t ns2 = fread ( s2, 2, 512, f2 );
        size_t i;

        if ( ns1 == 0 || ns2 == 0 || (ns1 != ns2) ) {
            if ( ferror(f1) ) {
                error++;
                printf ( "Error reading from '%s'\n", name1 );
            }
            if ( ferror(f2) ) {
                error++;
                printf ( "Error reading from '%s'\n", name2 );
            }
            if ( !error && (ns1 != ns2) ) {
                error++;
                if ( ns1 < ns2 ) {
                    printf ( "File '%s' is shorter than '%s'\n", name1, name2 );
                } else {
                    printf ( "File '%s' is longer than '%s'\n", name1, name2 );
                }
            }
            break;
        }

        for ( i = 0; i < ns1; i++ ) {
            if ( inverse ) {
                so[i] = s1[i] - s2[i];
            } else {
                so[i] = s1[i] + s2[i];
            }
        }

        if ( fwrite (so, 2, ns1, fo) < ns1 ) {
            error++;
            printf ( "Write error\n" );
            break;
        }
    }

    fclose ( f1 );
    fclose ( f2 );
    fclose ( fo );

    if ( !error ) {
        printf ( "done\n" );
    } else {
        remove ( nameo );
    }

    return 0;
}
